--local mouse = require "mouse"
--local gui = {}
 gui = {}
gui.error_msg_x = 10
gui.error_msg_y = 10
gui.help_text_x = 10
gui.help_text_y = gui.error_msg_y + gfx.texth
--///////////////////////
--// Scaling functions //
--///////////////////////

function scale_pos_to_slider_val(min_val, max_val, pos, min_pos, max_pos)
  local scaled_pos = min_val + (max_val - min_val) * (pos - min_pos) / (max_pos - min_pos)
  if scaled_pos > max_val then scaled_pos = max_val end
  if scaled_pos < min_val then scaled_pos = min_val end
  return scaled_pos
end


function scale_slider_val_to_x(min_val, max_val, slider_val, x1, x2)
  return (x1 + (slider_val - min_val) * (x2 - x1) / (max_val - min_val))
end


function scale_y_to_slider_val(min_val, max_val, y_coord, y1, y2)
  local scaled_y = min_val + (max_val - min_val) * (y_coord - y1) / (y2 - y1)
  if scaled_y > max_val then scaled_y = max_val end
  if scaled_y < min_val then scaled_y = min_val end
  return scaled_y
end


function scale_slider_val_to_y(min_val, max_val, slider_val, y1, y2)
  return y1 + (slider_val - min_val) * (y2 - y1) / (max_val - min_val)
end

--//////////////////
--// Slider class //
--//////////////////

local Slider = class(
              function(sl, x1, y1, w, h, val, default_val, min_val, max_val, lbl, help_text, id)
              --function(sl, ...)
                
                -- Slider position, w and h
                sl.x1 = x1 or 10
                sl.y1 = y1 or 10
                sl.w = w or 100
                sl.h = h or 15
                sl.x2 = sl.x1 + sl.w
                sl.y2 = sl.y1 + sl.h
                
                -- Mouse hit zone (Default = slider area)
                sl.hit_zone_x1 = sl.x1
                sl.hit_zone_y1 = sl.y1
                sl.hit_zone_w = sl.w
                sl.hit_zone_h = sl.h
                sl.hit_zone_x2 = sl.hit_zone_x1 + sl.hit_zone_w
                sl.hit_zone_y2 = sl.hit_zone_y1 + sl.hit_zone_h
                
                -- Slider values
                sl.min_val = min_val or 0
                sl.max_val = max_val or 1
                sl.default_val = default_val or 0.5
                sl.val = val or 0.5
        
                -- Scale slider value to x pos
                sl.pos = scale_slider_val_to_x(sl.min_val, sl.max_val, sl.val, sl.x1, sl.x2)
                sl.last_pos = sl.pos
                
                sl.y_coord = scale_slider_val_to_y(sl.min_val, sl.max_val, sl.val, sl.y1, sl.y2)
                sl.last_y_coord = sl.y_coord
                
                -- Label
                sl.lbl = lbl or ""
                
                sl.help_text = help_text or "" -- Help text is shown when mouse is hovering a slider
                
                --sl._mouse_state = false  -- true if mouse is clicked on "mouse hit zone"
                                         -- false when LMB or RMB is released
                
                sl.lbl_w, sl.lbl_h = gfx.measurestr(lbl) -- Label width and height
                
                sl.fg_r = 0.0
                sl.fg_g = 0.9
                sl.fg_b = 0.9
                sl.fg_a = 0.5
                
                sl.bg_r = 0.4
                sl.bg_g = 0.4
                sl.bg_b = 0.4
                sl.bg_a = 0.2
                
                sl.hz_r = 0.9
                sl.hz_g = 0.9
                sl.hz_b = 0.9
                sl.hz_a = 0.5
                
                
                
                -- Show/hide slider elements
                sl.show_label = true
                sl.show_value = true     -- Show/hide value
                sl.show_bg = true        -- Show/hide slider background
                sl.show_fg = true        -- Show/hide slider foreground
                sl.show_hit_zone = false -- Show/hide hit_zone
                
                sl.id = id or 0          -- this can be anything (f.ex. use 0 for first slider, 1 for second slider etc.) 
                --sl._dragging = false
                --sl.drag_sensitivity = 1               --  "fine adjustment factor"
                --sl.last_drag_sensitivity = sl.drag_sensitivity
                --Slider_ids[#Slider_ids+1] = sl.id
              end
              )
                    
Slider.last_touched = nil

function Slider:set_area(x1, y1, w, h)
  self.x1 = x1
  self.y1 = y1
  self.w = w
  self.h = h
  self.x2 = x1 + w
  self.y2 = y1 + h
end


function Slider:set_hit_zone(x1, y1, w, h)
  self.hit_zone_x1 = x1 or self.x1
  self.hit_zone_y1 = y1 or self.y1
  self.hit_zone_w = w or self.w
  self.hit_zone_h = h or self.h
  if x1~=nil and w~=nil then
    self.hit_zone_x2 = x1 + w
  else
    self.hit_zone_x2 = self.x1 + self.w
  end
  if y1~=nil and h~=nil then
    self.hit_zone_y2 = y1 + h
  else
    self.hit_zone_y2 = self.y1 + self.h
  end  
    
end


function Slider:set_help_text()
  if self.help_text == "" then return false end
  gfx.set(1,1,1,1)
  gfx.x = 10
  gfx.y = 10
  gfx.printf(self.help_text)
end


function Slider:set_fg_color(r,b,g,a)
  self.fg_r = r or gfx.r
  self.fg_g = g or gfx.g
  self.fg_b = b or gfx.b
  self.fg_a = a or gfx.a
end

function Slider:set_hz_color(r,b,g,a)
  self.hz_r = r or gfx.r
  self.hz_g = g or gfx.g
  self.hz_b = b or gfx.b
  self.hz_a = a or gfx.a
end

function Slider:set_bg_color(r,b,g,a)
  self.bg_r = r or gfx.r
  self.bg_g = g or gfx.g
  self.bg_b = b or gfx.b
  self.bg_a = a or gfx.a
end


function Slider:value_changed()
  return self.val ~= self.last_val
end


function Slider:mouse_on()
  if gfx.mouse_x > self.x1 and gfx.mouse_x < self.x2 and gfx.mouse_y > self.y1 and gfx.mouse_y < self.y2 then
    self._mouse_on = true
    return true
  else
    self._mouse_on = false
    return false
  end
  --return gfx.mouse_x > self.x1 and gfx.mouse_x < self.x2 and gfx.mouse_y > self.y1 and gfx.mouse_y < self.y2
end


function Slider:mouse_on_hitzone()
  if gfx.mouse_x > self.hit_zone_x1 and gfx.mouse_x < self.hit_zone_x2 and gfx.mouse_y > self.hit_zone_y1 and gfx.mouse_y < self.hit_zone_y2 then
    self._mouse_on_hitzone = true
    --self._mouse_state = true
    return true
  else
    self._mouse_on_hitzone = false
    return false
  end
  --return gfx.mouse_x > self.x1 and gfx.mouse_x < self.x2 and gfx.mouse_y > self.y1 and gfx.mouse_y < self.y2
end


function Slider:get_val()
  return self.val
end


function Slider:set_val(val)
  self.val = val
end


function Slider:get_default_val()
  return self.default_val
end


function Slider:show_foreground(b_show)
  self.show_fg = b_show
end


function Slider:show_background(b_show)
  self.show_bg = b_show
end


function Slider:show_hitzone(b_show)
  self.show_hit_zone = b_show
end


function Slider:draw_value(x, y, digits_after_dec_point, override_val)
  if self.show_value then
    local digits = digits_after_dec_point or 3
    local val_str = "%." .. tostring(digits).. "f"
    local val = override_val or self.val
    val_str = string.format(val_str, val)
    self.val_w = gfx.measurestr(val_str)
    gfx.set(0.9,0.9,0.9,1) 
    gfx.x = x or self.x2-self.val_w - 2
    gfx.y = y or self.y1
    gfx.printf(val_str)
  end
end


function Slider:draw_label(x, y)
  -- Draw slider label (if "slider_label" is not an empty string)
  if self.show_label then 
    if self.lbl ~= "" then
      gfx.x = x or self.x1+4
       --gfx.x = 0.5*(self.x1+self.x2)-0.5*self.lbl_w
      gfx.y = y or self.y1 + 0.5*self.h - 0.5*gfx.texth
      gfx.set(1,1,1,1);
      gfx.printf(self.lbl)
    end
  end
end


function Slider:update()
 
  --end
  --mouse.dx = gfx.mouse_x - mouse.ox_l
  --[[
  if self.drag_sensitivity ~= self.last_drag_sensitivity then
    self.last_pos = self.pos-self.drag_sensitivity*mouse.dx
    self.last_drag_sensitivity = self.drag_sensitivity
  end
  --]]
  
  --gfx.set(0.2,0.7,0.7,self.a)
  
  -- Update slider position (foreground) when slider value changes
  if self:value_changed() then
    self.pos = scale_slider_val_to_x(self.min_val, self.max_val, self.val, self.x1, self.x2)
  end
  
  if mouse.last_LMB_state == false and self._dragging then 
    self._dragging = false
  end
  
  if not self._dragging then
    self.last_pos = self.pos
  end
  
  if self._dragging or self:mouse_on_hitzone() then
    if self._dragging or mouse.last_LMB_state == false then self:set_help_text() end
    if mouse.last_LMB_state == false and mouse.cap(mouse.LB) and not self._dragging then
      self._dragging = true
      Slider_last_touched = self.id
      --end
    end
    if self._dragging then
      --[[
      -- Set drag sensitivity
      local cap = mouse.cap()
      if cap == mouse.CTRL + mouse.LB then
        self.drag_sensitivity = 0.25 
      elseif cap == mouse.CTRL + mouse.SHIFT + mouse.LB then
        self.drag_sensitivity = 0.1
      else 
        self.drag_sensitivity = 0.5
      end
      --]]
      
      -- Update slider position
      self.pos = math.min(math.max(self.x1, self.last_pos+self.drag_sensitivity*mouse.dx),self.x2)
    end
  end

  self.val = scale_pos_to_slider_val(self.min_val, self.max_val, self.pos, self.x1, self.x2)
  
 
  -- Draw slider foreground
  if self.show_fg then
    gfx.set(self.fg_r, self.fg_g, self.fg_b, self.fg_a)
    gfx.rect(self.x1, self.y1, self.pos - self.x1, self.h)
  end
  
  -- Draw slider background
  if self.show_bg then 
    gfx.set(self.bg_r,self.bg_g,self.bg_b,self.bg_a)
    gfx.rect(self.x1, self.y1, self.w, self.h)
  end
    
  -- Draw mouse hit zone
  if self.show_hit_zone then
    gfx.set(self.hz_r,self.hz_g,self.hz_b,self.hz_a)
    gfx.rect(self.hit_zone_x1, self.hit_zone_y1, self.hit_zone_w, self.hit_zone_h)
  end
  
  
end


function Slider:set_to_default_value()
  self.val = self.default_val
  --self:update()
end


return Slider, Slider_last_touched
